@@ -178,10 +178,8 @@ sub is_valid_tree {
 ###############
 
 sub _is_unicode {
-  if ($] >= 5.8) {
-    eval(q`
-      if ( $_[0] =~ /[\x{100}-\x{10FFFF}]/s) { return 1 ;}}
-    `);
+  if ($] >= 5.008) {
+    if ( utf8::is_utf8($_[0])) { return 1 ;}
   }
   else {
     ## No Perl internal support for UTF-8! ;-/
